/*=============================================================================
vitalnix/accdb - User Management Suite - Unified Account Database
accdb.h - Include file for ACCDB back-end mod.s or applications utilizing them
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
#ifndef _ACCDB_H
#define _ACCDB_H 1

#ifdef __cplusplus
extern "C" {
#endif

struct accdb_user {
    // passwd part
    char *lname;
    long uid, gid;
    char *gecos, *home, *shell, *igrp, *sgrp;

    // shadow part
    char *passwd;
    long last_change, keep_min, keep_max, warn_age, expire, inactive;
    void *_private;
};

struct accdb_group {
    long gid;
    char *gname;
    /// XXX gshadow part??
    void *_private;
};

struct accdb_module {
    const char *name, *desc, *info;
    void *handle, *state;
    int (*open)(void **);
    int (*usertrav)(void *, struct accdb_user *);
    int (*userinfo)(void *, struct accdb_user *, struct accdb_user *, size_t);
    int (*useradd)(void *, struct accdb_user *);
    int (*usermod)(void *, struct accdb_user *, struct accdb_user *);
    int (*userdel)(void *, struct accdb_user *);
    int (*grouptrav)(void *, struct accdb_group *);
    int (*groupinfo)(void *, struct accdb_group *, struct accdb_group *,
     size_t);
    int (*groupadd)(void *, struct accdb_group *);
    int (*groupmod)(void *, struct accdb_group *, struct accdb_group *);
    int (*groupdel)(void *, struct accdb_group *);
    int (*close)(void *);
    int (*sysctl)(unsigned int, ...);
};

struct rconfig_opt {
    const char *key;
    char type;
    void *ptr;
    int (*callback)(const char *, char, void *);
};

enum accdb_genpw_meth {
    PW_DES = 0,
    PW_MD5,
    PW_BLOWFISH,
};

enum {
    PW_PHONEMIC  = 1 << 0,
    PW_ONE_DIGIT = 1 << 1,
    PW_ONE_CASE  = 1 << 2,
};

enum {
    RCONFIG_ONE  = 1 << 0, // only read one configuration file
};

enum {
    ACCDB_NOAUTOFLUSH = 1 << 0,
};

enum accdb_sysctl_req {
    ACCDB_ADDFLAGS = 0x1,
    ACCDB_DELFLAGS,
    ACCDB_FLUSHDB,
    ACCDB_COUNT_USERS,
    ACCDB_COUNT_GROUPS,
    ACCDB_NEXTUID_SYS = 0x100,
    ACCDB_NEXTUID,
    ACCDB_NEXTGID_SYS,
    ACCDB_NEXTGID,
    // module specific sysctl requests shall start at 0xC000
};

struct accdb_module *accdb_load(char *);
void accdb_unload(struct accdb_module *);
int accdb_genpw(char *, char **, size_t, unsigned long, unsigned long);
extern int accdb_rconfig(char *, struct rconfig_opt *);
extern void accdb_rconfig_pv(const char **, char *, struct rconfig_opt *,
 unsigned long);
extern void accdb_rconfig_free(struct rconfig_opt *);

#ifdef __cplusplus
} // extern "C"
#endif

#endif /* _ACCDB_H */

//==[ End of file ]============================================================
